PROGRAM_NAME='RaneRPM'

// System Type    : Netlinx

// FILE CREATED ON: 9/12/2002 11:06PM
//              BY: 3rd Party Systems
//                  5316 91st Ave SE
//                  Snohomish, WA  98290
//                  (360) 568-9522
//                  www.thrdParty.com
// FILE REVISION  : Rev 1.0
// REVISION DATE  : 1/10/2003 8:48PM

// A Standard command to the RPM unit consists of 23 bytes.  Obviously, these bytes
// must be sent in a specific order.  Following is an explanation of the way these
// bytes are ordered, and the way they are addressed in this include file.
// 
// There are two CHAR strings defined in the variable section.  One is the string
// that is created for sending a command to the RPM unit, rpmCmdStr[24], and the 
// other is a string for receiving responses back from the RPM, rpmRetStr[24].
// 
// When sending a command to the RPM, you must set parameters that make sense to the
// RPM.  The 23 bytes of a command are as follows:
//      Position    Description/Constant    Explanation
//              1           Operation                           There are 3 options here.  
//                          (bpOP)                              0 = Get - get a level, selection channel or
//                                                                              any other parameter associated with a
//                                                                              specific object.  
//                                                                              (RPM_GET)
//                                                                      1 = Set - set a level, channel or other
//                                                                              parameter associated with an object.
//                                                                              (RPM_SET)
//                                                                      2 = Response - this is returned from the
//                                                                              RPM.  When the first byte of the return
//                                                                              string = 2, we will check the error fields
//                                                                              and data fileds to get the values we are
//                                                                              interested in.
//                                                                              (RPM_RESP)
//           2-3        Block Error Code        This byte, and the next will contain an error
//                            (bpBLKERR)                    code in the return string if there was a problem.
//                                                                      We will set these two bytes to equal 0 in the send
//                                                                      string prior to sending the command.
//              4           Memory Number                   Setting this byte to 0 sends and receives commands
//                          (bpMEMNUM)                      to the current working memory.  Further explanation
//                                                                      of using Memory numbers will be found in seperate
//                                                                      documentation.
//           5-8        MAP ID  (bpMAPID)           Future implementation.  Currently, these 4 bytes must 
//                                                                      be set to 0.
//              9           Number of                           It is possible to send more than 1 command in a single
//                          Block References            Send string, however this include file does not support 
//                          (bpNUMBLKS)                     this feature. If you wish to use it, the declarations 
//                                                                      for rpmCmdStr[] and rpmRetStr[] must be increased 
//                                                                      from 23.
//                          Can also be thought     Each command section of a Send string is 14 bytes. The 
//                          of as Number of             header portion is 9 bytes.  Therefore, multiple 
//                          Commands                            commands in a single Send string would be: 
//                                                                      (14 * Number of Cmds) + 9   bytes in length.  This file
//                                                                      sets this value to 1.                                                                       
//          10-11       Block PID                           This is a unique number for each object in the RPM
//                          (bpPID)                             processing map.  You get this number from the Drag-Net
//                                                                      program.  The Drag-Net program gives the number as a
//                                                                      decimal value.  This include file has a Define_Call,
//                                                                      GetDWord that will set the two Block PID bytes to the
//                                                                      proper hex value.  This is described in better detail
//                                                                      later in this document.
//          12-13       Block BType                     Each object, or Block, has a type number.  This number
//                          (bpBTYPE)                           is given in the Drag-Net Script utility. Currently the 
//                                                                      BType is coded into each of the Functions.  This could 
//                                                                      be changed to a table type of entry by creating an 
//                                                                      array with the BType of each type of Block used.
//          14-15       Block 'n' Parameter     Each Block has several functions, or Parameters.  For
//                              Index   (bpIDX)             instance, a level control has a Trim and a Mute 
//                                                                      function (as well as others).
//                                                                      The Mute function has an index value of 3, the Trim
//                                                                      function has a value of 2.  These 2 Bytes are the 
//                                                                      index value given in the Drag Net report.  These two
//                                                                      bytes are coded in each function.
//          16-17       Block 'n' Error             These two bytes must be set to 0 in the rpmCmdStr[].
//                          Codes  (bpCERR)             A non-zero value in the rpmRetStr[] indicates there 
//                                                                      was an error with this command.  Remember that more
//                                                                      than 1 command can be sent.  Each   command block will
//                                                                      have it's own error bytes.
//          18-19       Block 'n' Parameter     This value is the number of bytes of data for the 
//                                                                      command to be executed.  This value is also coded
//                          Length  (bpNUMBYTES)    into each function. The functions in this file only 
//                                                                      use 1 byte of data for each command.  
//          20-23       Block 'n' Values            These four bytes pass the data values needed for the
//                              (bpARGS)                        parameter being called.  If a parameter requires 
//                                                                      only one byte of data, rpmCmdStr[bpArgs + 3] would
//                                                                      contain that value.

//      *******  CONSTANT DEFINITIONS GO BELOW  *******
DEFINE_CONSTANT
// 
RPM_GET = 0   RPM_SET = 1   RPM_RESP = 2
// port that is used for the UDP 2-way conection
RPM_PORT = 5000

// The following value will add or subtract 3 to the current volume value.  That value
// will be between 0 and 255.  Different devices within the RPM have different step 
// values for the volume.  The Level that we are dealing with increments/decrements
// in .5dB steps.  Therefore, we will increment/decrement in 1.5dB steps with this
// declared value of 3.  Some volume controls will icrement/decrement in 1dB, .5dB or
// .25dB steps.  You may want to declare different INC_Val constants for the 
// different types.
RPM_INC_VAL             = 3


// byte positions for commands - as expained above
bpOP   = 1   bpBLKERR = 2       bpMEMNUM = 4    bpMAPID = 5   bpNUMBLKS   = 9
bpPID  = 10  bpBTYPE  = 12  bpIDX    = 14   bpCERR  = 16  bpNUMBYTES = 18
bpARGS = 20

// Level Control function args
RPM_MUTE = 1  RPM_UNMUTE = 2  RPM_INCREMENT = 3   RPM_DECREMENT = 4   RPM_ABSOLUTE = 5
RPM_LEV_MIN = 6     RPM_LEV_MAX = 7     RPM_OFFMIN = 8

//     *******  DATA TYPE DEFINITIONS GO BELOW   ********

DEFINE_TYPE

// these structures are used in the Define_Variable section when declaring a variable
// associated with a Block, or device/object.  When declaring a variable, use these
// structures as the Type, and assign the values for PID, Initial Volume Level, and
// whether the Mute is on/off. Example:
//
// Define_Variable  (section in main program)
// RPM_LEVEL       L_Lvl   = {21,192,0}    // Level 1 on left channel just prior to output 1
// RPM_LEVEL       R_Lvl   = {26,192,0}    // Level 6 on right channel just prior to output 2  
//
// This creates a variable called L_LVL of type RPM_LEVEL.  Here we assign the PID (21),
// Initial volume level (192 = 0dB for a Block that uses .5dB increments), and Mute
// status (0 = Mute Off).  The PID is assigned by Drag Net and can be found in the Block
// Report.  The Same is true for the creation of R_LVL, except we set it's PID to 26.

structure RPM_LEVEL {
  integer pid       // PID given in Drag Net Report
  integer vol       // Current volume level
  integer mute  // If true, Mute is on, else mute is off
  integer   min     // Min value for volume
  integer max       // Max value for volume
  char      offAtMin    // If true, Mute when volume hits min value, 
}

// define a 1 of N Select in the same manner.  
//
// Define_Variable  (section in main program)
// RPM_SELECT      L_SEL   = {31,3,1} // 3 channel select for output 1
//                                                                          (pid, # of chans, current chan)
// RPM_SELECT      R_SEL   = {33,4,1} // 4 channel select for output 2
//
// This creates a couple of variables of Type RPM_Select.  We set the initial values during
// the declaration.  L_SEL gets initialized with the PID=31, it is a 1 of 3 Select, and  
// we want to choose chan 1 as the channel to be selected on startup.
 
structure RPM_SELECT {
  integer pid
  integer chans
  integer currChan
}

// This structure is not used in this program.  As mentioned above, this would allow multiple 
// commands to be sent in one packet.  You could create a variable of this Type and use it to
// send the multiple command packet to the RPM.  The SendRPMCommand() function would need to
// be modified to send the multi-command string and parse the response to check all error
// fields.
structure RPM_CMD {
  char    sCmd[65]  // enough storage for up to 4 commands to a single block at a time
  integer count
}


//  ***********  VARIABLE DEFINITIONS GO BELOW  *********

DEFINE_VARIABLE

rpmOpen
char rpmCmdStr[23]      // the 23 byte string sent to the RPM
char rpmRetStr[23]      // the 23 byte response string from the RPM

//                  DANGER!! DANGER!! DANGER!!  ----- READ!! READ!! READ!!  READ!!
// The next two variables should not be changed unless really, really, really necessary!
// The rpmWaitLen is in 1/10th of a second.  On a busy network, you may increase this 
// value to allow more time for the reutrn of a valid response, 
//                          BUT DO NOT INCREASE ABOVE A VALUE OF 5!!!!!
// If you do not get a response within 1/2 sec., even on a busy network, something is WRONG!
//
// The rpmRetry is the number of times the command will be resent if a valid repsone is not
// received within the time: rpmWaitLen.  Please see the explanation in the SendRPMCommand() 
// function for more on the dangers of changing these values willy-nilly.
// End of DANGER!! DANGER!! DANGER!!
char rpmWaitLen = 2   // change this val to lengthen wait time for command verification from RPM
char rpmRetry = 5           // This is the number of times the command will be resent before returning a fail

//  ***********  Include file delcarations      **********


//  ***********  LATCHING DEFINITIONS GO BELOW  **********
DEFINE_LATCHING



// *******  MUTUALLY EXCLUSIVE DEFINITIONS GO BELOW ******
DEFINE_MUTUALLY_EXCLUSIVE


// ******  SUBROUTINE/FUNCTION DEFINITIONS GO BELOW  *****

// This Call will take an integer value and convert it to a 2-byte value.  The first
// argument, 'tval', is the integer value you want converted.  The next two arguments
// are the two CHAR variables to store the result in.  Typically these will be the PID bytes
// in the rpmCmdStr: rpmCmdStr[bpPID] and rpmCmdStr[bpPID + 1].
Define_Call 'GetDword'(integer tval, char byte1, char byte2) 
  Local_Var tStr[4]
{
  if(tval > 255) {
    tStr = ItoHex(tval)
    if(Length_String(tStr) < 4)
        byte1 = hextoi(Left_String(tStr,1))
    else
        byte1 = hextoi(Left_String(tStr,2))
        
    byte2 = hextoi(Right_String(tStr,2))
  }
  else {
    byte1 = 0
    byte2 = TYPE_CAST(tval)
  }
}
//-----------------------------------------------------------------------------------------------
// ATTENTION!!!
// This function is basically the heart of the functionality of talking to the RPM.
// After you have set all of the bytes in the rpmCmdStr[], then you would call this function.
// This function sends the rpmCmdStr[] to the RPM and waits for a response from the RPM.  
// Step-by-step, this is what happens:
// When called, the function 
//  1) enters a long_while loop that will iterate rpmRetry (5) times before returning fail.  
//  2) we turn the time-out variable off
//  3) set a Wait for the time specified by rpmWaitlen (2/10 sec.)  When 2/10 sec is up, 
//       time-out is true
//  4) blank out the response string
//  5) send the rpmCmdStr[] that was previously set with the values we wanted
//  6) enter another long_while loop that will terminate when we receive a response
//       or when the 2/10 sec expires.
//  7) if  a string that is the same length isreceived back, we check all the error byte 
//       positions. If all of them are 0, command was OK and we exit function
//  8) If the time-out (2/10 sec) happens, we resend the command  and start that loop again
//  9) If after 5 tries we don't get a valid response we return 0, a failure

// There is a drawback to this routine.  When sending a command to the RPM, if there are 
// errors on the network and a command must be resent, 2/10 of a second is chewed up.  If
// a command must be sent 5 times, (the max retry value), one second is consumed.  
// During that second, the rest of the system will be unresponsive.  If there are errors 
// during a ramping routine, which sends many commands in a second, the system can appear 
// to be locked.  Let's say that during a ramp routine 50 commands were sent.  If there 
// is a 10% failure rate receiving UDP packets, 5 seconds will be needed and the system 
// cannot process other commands during that time.

// If you are doing a ramp routine, it is recommended that you use the 
// SendRPMCommand_NoProc() function.  This function is called in the same way as the 
// SendRPMCommand() function, but it does not check for a response from the RPM unit.  
// In a ramping situation this is generally not a problem.

Define_Function char SendRPMCommand() {
  Stack_Var   idx
  Local_Var   tmOut

  idx = 0
  long_while(idx < rpmRetry) {
    off[tmOut]
    wait rpmWaitLen
      on[tmOut]
    rpmRetStr = ''
    Send_String dvRPM_CLIENT, "rpmCmdStr"
    Long_While((Length_String(rpmRetStr) < 1) && !tmOut) {
    }
    if(Length_String(rpmRetStr) == Length_String(rpmCmdStr)) {
      if((rpmRetStr[bpBLKERR] == 0) && (rpmRetStr[bpBLKERR+1] == 0) 
        && (rpmRetStr[bpCERR] == 0) && (rpmRetStr[bpCERR+1] == 0)) {
        return(1)   // successful tx/rx
      }
    }
    #if_defined debug
      send_string dvDEBUG,"'Sending Command ',itoa(idx+1), ' times for block: ',13,10"
    #end_if
    idx++
  }
  return(0)   // failed tx/rx
}
//-----------------------------------------------------------------------------------------------

// This function sends a command to the RPM unit but does not check for a response
// to verify a good tranmission.  This would be the preferred way to send commands to
// the RPM in a ramping type situation, or anytime you are sending a large amount of
// commands in a non-critical situation.  If you must have verification that a command
// was processed, use the SendRPMCommand() function above.

// This function takes an argument so that you may use the same command for multiple
// units.  When calling this function pass the device that you declared in the Define_Device
// section of your main program.
// ie: SendRPMCommand_NoProc(dvRPM_Client) or SendRPMCommand_NoProc(dvRPM_Client2)
Define_Function char SendRPMCommand_NoProc(DEV rpmUnit) {

  Send_String rpmUnit, "rpmCmdStr"

  #if_defined debug
    send_string dvDEBUG,"'Sending Command ',rpmCmdStr,13,10"
  #end_if
  return(1)   // tx OK
}
//-----------------------------------------------------------------------------------------------

// This function calls a preset.  The only argument is the number of the preset you want recalled.
// DO NOT MODIFY THIS FUNCTION!
Define_Function char RecallRPMPreset(integer pNum) {
  Local_Var   idx
  
  idx = 1
  // We set each byte of rpmCmdStr[] to the proper value.  
  rpmCmdStr[bpOP] = RPM_SET     rpmCmdStr[bpNUMBLKS] = $01
  rpmCmdStr[bpPID] = $04        rpmCmdStr[bpPID + 1] = $01
  rpmCmdStr[bpBTYPE] = $FC      rpmCmdStr[bpBTYPE + 1] = $00  
  rpmCmdStr[bpIDX] = $00        rpmCmdStr[bpIDX + 1] = $00
  rpmCmdStr[bpNUMBYTES] = $00   rpmCmdStr[bpNUMBYTES + 1] = $01
  rpmCmdStr[bpARGS] = $00       rpmCmdStr[bpARGS + 1] = $00
  rpmCmdStr[bpARGS + 2] = $00   rpmCmdStr[bpARGS + 3] = pNum

  #if_defined debug
    send_string dvDEBUG,"'Recalling Preset: ',itoa(pNum),13,10"
  #end_if

  if(SendRPMCommand()) {
    return(1)
  }
  else {
    return(0)    
  }
}
//-----------------------------------------------------------------------------------------------
// This function returns the value of the volume, or status of the mute for a Level control.
// Arguments are the Level to be controlled, and either RPM_MUTE or RPM_INCREMENT
// ie: volValue = GetRPMLevel(L_LVL, RPM_INCREMENT)

// DO NOT MODIFY THIS FUNCTION!
Define_Function integer GetRPMLevel(RPM_LEVEL lev, char func) {

  rpmCmdStr[bpOP] = RPM_GET     rpmCmdStr[bpNUMBLKS] = $01
  rpmCmdStr[bpBTYPE] = $01      rpmCmdStr[bpBTYPE + 1] = $80  
  rpmCmdStr[bpNUMBYTES] = $00   rpmCmdStr[bpNUMBYTES + 1] = $01
  rpmCmdStr[bpARGS] = $00       rpmCmdStr[bpARGS + 1] = $00
  rpmCmdStr[bpARGS + 2] = $00   rpmCmdStr[bpARGS + 3] = $00

  // split the pid into 2 bytes if necessary  
  Call 'GetDword'(lev.pid, rpmCmdStr[bpPID], rpmCmdStr[bpPID + 1])
  switch(func) {
    case RPM_MUTE: {
        // the bpIDX value is shown in the Drag Net Report for your blocks. In this case 3 = Mute
      rpmCmdStr[bpIDX] = $00
      rpmCmdStr[bpIDX + 1] = $03
      rpmCmdStr[bpARGS + 3] = $00
      break;
    }
    case RPM_INCREMENT: {
      rpmCmdStr[bpIDX] = $00
      rpmCmdStr[bpIDX + 1] = $02
      rpmCmdStr[bpARGS + 3] = $00
    }
    }
  
  if(SendRPMCommand()) {
    switch(func) {
        case RPM_MUTE:          lev.mute = rpmRetStr[bpARGS + 3]
        case RPM_INCREMENT: lev.vol = rpmRetStr[bpARGS + 3]
    }
    return(1)
  }
  return(0)    

}
//-----------------------------------------------------------------------------------------------
// This function sets the value of the volume, or status of the mute for a Level control.
// Arguments are Level to be controlled, and one of: RPM_MUTE, RPM_INCREMENT
// RPM_DECREMENT or RPM_ABSOLUTE.  In the case of RPM_Absolute, set the value of volume
// prior to calling this function.  ie: 
//  L_LVL.vol = 128  
//  SetRPMLevel(L_LVL, RPM_Absolute)

// DO NOT MODIFY THIS FUNCTION!
Define_Function integer SetRPMLevel(RPM_LEVEL lev, char func) {
  
  rpmCmdStr[bpOP] = RPM_SET     rpmCmdStr[bpNUMBLKS] = $01
  rpmCmdStr[bpBTYPE] = $01      rpmCmdStr[bpBTYPE + 1] = $80  
  rpmCmdStr[bpNUMBYTES] = $00   rpmCmdStr[bpNUMBYTES + 1] = $01
  rpmCmdStr[bpARGS] = $00       rpmCmdStr[bpARGS + 1] = $00
  rpmCmdStr[bpARGS + 2] = $00

  // split the pid into 2 bytes if necessary  
  Call 'GetDword'(lev.pid,rpmCmdStr[bpPID],rpmCmdStr[bpPID + 1])

  switch(func) {
    case RPM_MUTE: {
      rpmCmdStr[bpIDX] = $00
      rpmCmdStr[bpIDX + 1] = $03
      rpmCmdStr[bpARGS + 3] = 1
    }
    case RPM_UNMUTE: {
      rpmCmdStr[bpIDX] = $00
      rpmCmdStr[bpIDX + 1] = $03
      rpmCmdStr[bpARGS + 3] = 0
    }
    case RPM_INCREMENT: {
      rpmCmdStr[bpIDX] = $00
      rpmCmdStr[bpIDX + 1] = $02
      if(lev.vol < 255)
        lev.vol = lev.vol + RPM_INC_VAL
      rpmCmdStr[bpARGS + 3] = TYPE_CAST(lev.vol)
    }
    case RPM_DECREMENT: {
      rpmCmdStr[bpIDX] = $00
      rpmCmdStr[bpIDX + 1] = $02
      if(lev.vol > 0)
        lev.vol = lev.vol - RPM_INC_VAL
      rpmCmdStr[bpARGS + 3] = TYPE_CAST(lev.vol)
    }
    case RPM_ABSOLUTE: {
      rpmCmdStr[bpIDX] = $00
      rpmCmdStr[bpIDX + 1] = $02
      rpmCmdStr[bpARGS + 3] = TYPE_CAST(lev.vol)
    }
  }

  #if_defined debug
    send_string dvDEBUG,"'Setting Volume/Mute on Level: ',itoa(lev.pid),13,10"
  #end_if

  if(SendRPMCommand()) {
    switch(func) {
      case RPM_MUTE: {
        lev.mute = 1
      }
      case RPM_UNMUTE: {
        lev.mute = 0
      }
      case RPM_INCREMENT: {
                lev.vol = rpmRetStr[bpARGS + 3]
      }
      case RPM_DECREMENT: {
                lev.vol = rpmRetStr[bpARGS + 3]
      }
      case RPM_ABSOLUTE: {
                lev.vol = rpmRetStr[bpARGS + 3]
      }
    }
      return(1)
  }
  return(0)    
}
//-----------------------------------------------------------------------------------------------
// This function sets the active input channel for a 1-of-N Select control.
// Argument is the Select block to control.  In your program, set the Select to the channel
// you want and call the function
//  CH1_SEL.currChan = 2  
//  SetRPMSelect(CH1_SEL)

// DO NOT MODIFY THIS FUNCTION!

Define_Function integer SetRPMSelect(RPM_SELECT sel) {
  Local_Var char    idx
            integer tInt
  
  idx = 1
  rpmCmdStr[bpOP] = RPM_SET     rpmCmdStr[bpNUMBLKS] = $01
  rpmCmdStr[bpIDX] = $00        rpmCmdStr[bpIDX + 1] = $02
  rpmCmdStr[bpNUMBYTES] = $00   rpmCmdStr[bpNUMBYTES + 1] = $01
  rpmCmdStr[bpARGS] = $00       rpmCmdStr[bpARGS + 1] = $00
  rpmCmdStr[bpARGS + 2] = $00   rpmCmdStr[bpARGS + 3] = TYPE_CAST(sel.currChan)

  // split the pid into 2 bytes if necessary  
  Call 'GetDword'(sel.pid,  rpmCmdStr[bpPID],   rpmCmdStr[bpPID + 1])
  
  tInt = 528 + sel.chans - 2  // base BType for a 1X2 selector is 528
                              // increases by 1 for each additional channel
  // split the BType into 2  
  Call 'GetDword'(tInt,rpmCmdStr[bpBTYPE],rpmCmdStr[bpBTYPE + 1])

  #if_defined debug
    send_string dvDEBUG,"'Selecting Chan: ',itoa(sel.currChan),' on Selector: ',itoa(sel.pid),13,10"
  #end_if

  if(SendRPMCommand()) {
    return(1)
  }
  return(0)    
}
//-----------------------------------------------------------------------------------------------

Define_Function integer GetRPMSelect(RPM_SELECT sel) {
  Local_Var char    idx
            integer tInt
  
  idx = 1
  rpmCmdStr[bpOP] = RPM_GET     rpmCmdStr[bpNUMBLKS] = $01
  rpmCmdStr[bpIDX] = $00        rpmCmdStr[bpIDX + 1] = $02
  rpmCmdStr[bpNUMBYTES] = $00   rpmCmdStr[bpNUMBYTES + 1] = $01
  rpmCmdStr[bpARGS] = $00       rpmCmdStr[bpARGS + 1] = $00
  rpmCmdStr[bpARGS + 2] = $00   rpmCmdStr[bpARGS + 3] = $00

  // split the pid into 2 bytes if necessary  
  Call 'GetDword'(sel.pid,rpmCmdStr[bpPID],rpmCmdStr[bpPID + 1])
  
  tInt = 528 + sel.chans - 2  // base BType for a 1X2 selector is 528
                              // increases by 1 for each additional channel
  // split the BType into 2  
  Call 'GetDword'(tInt,rpmCmdStr[bpBTYPE],rpmCmdStr[bpBTYPE + 1])

  #if_defined debug
    send_string dvDEBUG,"'Selecting Chan: ',itoa(sel.currChan),' on Selector: ',itoa(sel.pid),13,10"
  #end_if

  if(SendRPMCommand()) {
    sel.currChan = rpmRetStr[bpARGS + 3]
    return(1)
  }
  return(0)    
}
//-----------------------------------------------------------------------------------------------


// *************  STARTUP CODE GOES BELOW  ***************

DEFINE_START

// Do Not change this line - sets necessary bytes to zero
rpmCmdStr = "$01,$00,$00,$00,$00,$00,$00,$00,$01,$04,$01,$FC,$00,$00,$00,$00,$00,$00,
                                $01,$00,$00,$00,$00"


// This is opening the ethernet port for the RPM unit using UDP 2-way protocol
// You must have this function for each RPM unit in your system.
// A second declaration would be something like:
//          IP_Cient_Open(dvRPM_CLIENT2.Port,RPM_ADDR2,RPM_PORT2,UDP_2)
IP_Client_Open(dvRPM_CLIENT.Port,RPM_ADDR,RPM_PORT,UDP_2)

on[rpmOpen]
#if_defined debug
  send_string 0:0:0,"'RPM device Online',13,10"
#end_if


//  *************  THE EVENTS GOES BELOW  ****************

DEFINE_EVENT

//  ****************  Data Events  **************************

// If you have multiple RPM units, you will set up a Data_Event for each one

Data_Event[dvRPM_CLIENT] {  // data recv'd from RPM unit
  online: {
    #if_defined debug
      send_string 0:0:0,"'RPM Online',13,10"
    #end_if
    on[rpmOpen]
  }
  offline: {
    #if_defined debug
      send_string 0:0:0,"'Device Offline',13,10"
    #end_if
    ip_client_close(dvRPM_CLIENT.Port)
    off[rpmOpen]
  }
  String: {
    #if_defined debug
      send_string 0:0:0,"'RPM Return String: ',data.text,13,10"
    #end_if
    rpmRetStr = Data.Text

  }     // String
}


//  ***********  THE ACTUAL PROGRAM GOES BELOW  **********

DEFINE_PROGRAM

//  ****************   END OF PROGRAM  *******************

